""" 
    Title: 
        OBJ Based Classification 
 
    Description: 
        Object Based Classification using Multiband Imagery and Point class to classify vegetation type. This tool
        also calculates the classifications Cell density of value "5"(Juniper) This density is then visualized using a fishnet to
        identify the Juniper phase.
 
    Limitations: 
        Points feature Class has to be located in the Classifcation Geodatabase located in "..\Classification\ToolData\Classification.gdb".
        This allows the "Class" Domain to be applied to the VegType field. This domain controls the Coded Values. 
 
    Authors: 
        NASA DEVELOP - BLM at ISU GIS TREC
        Ryan Howerton (howeryan@isu.edu) 
        Jenna Williams (will55200@gmail.com)
        
    Created: 
        4/8/16 
"""
import arcpy
import os, sys, shutil

DIR = os.path.dirname(os.path.realpath(__file__)) + os.sep
DIR = DIR.rstrip('\Tool\\')
arcpy.CheckOutExtension('Spatial')
arcpy.env.overwriteOutput = True
sr = arcpy.SpatialReference(r"" + DIR + "\\ToolData\\WGS_1984_UTM_Zone_12N.prj")
SCRAP = DIR + "\\Scrap\\"
arcpy.env.workspace = (SCRAP)
arcpy.env.outputCoordinateSystem = DIR + "\\ToolData\\WGS_1984_UTM_Zone_12N.prj"
clr = DIR + "\\ToolData\\Classification_Colors.clr"
clrFN = DIR + "\\ToolData\\Fishnet.lyr"
mxd = arcpy.mapping.MapDocument("current")  
df = arcpy.mapping.ListDataFrames(mxd)[0]  
df.spatialReference = sr 

# Input Variables
MapService = arcpy.GetParameterAsText(0)
Shp = arcpy.GetParameterAsText(1)
name = arcpy.GetParameterAsText(2)
TrainNAIPPoints = arcpy.GetParameterAsText(3)
OUTLocation = arcpy.GetParameterAsText(4)


#Outnames
OUTLocation = OUTLocation + "\\"
SCRAPLocation = SCRAP
ImageryOutName = str(SCRAPLocation) + str(name) + "_Imagery.tif"
segoutname = str(SCRAPLocation) + str(name) + "_SEG.tif"


# Process: Subset Points 60%-Training 40%-Validation 
inPointFeatures = TrainNAIPPoints
outtrainPoints = str(OUTLocation) + str(name) + "_Train_Points.shp"
outValidationPoints = str(SCRAPLocation)+ str(name) + "_Validation_Points_IN.shp"
trainData = "60" 
subsizeUnits = "PERCENTAGE_OF_INPUT"
arcpy.SubsetFeatures_ga(inPointFeatures, outtrainPoints, outValidationPoints,trainData, subsizeUnits)

# Process: Extract by Mask
arcpy.gp.ExtractByMask_sa(MapService, Shp, ImageryOutName)
arcpy.env.workspace = (SCRAPLocation)
# Process: Segment Mean Shift
arcpy.gp.SegmentMeanShift_sa(ImageryOutName,segoutname, "20", "20", "20", "")

# Process: Create Signatures
List = []
i = 1
Band = "B"+str(i)
Band = str(";"+str(ImageryOutName) + "\\Band_" + str(i))
BandList = [segoutname]
for line in BandList:
    if arcpy.Exists(line):
        arcpy.AddMessage(line)
        List.append(line)
        Band = str(str(ImageryOutName) + "\\Band_" + str(i))
        i += 1
        BandList.append(Band)
    else:
        arcpy.AddMessage(str(line)+ " - Does Not Exist")

arcpy.env.workspace = (SCRAP)
Signature = ("r"+ str(name)+ "_Signature.gsg")
arcpy.gp.CreateSignatures_sa(List, outtrainPoints, Signature, "COVARIANCE", "VegType")


# Process: Maximum Likelihood Classification
ClassifiedSCR = str(SCRAPLocation) + str(name)+"_Classification.tif"
ConfidenceSCR = str(SCRAPLocation) + str(name)+"_ClassConf.tif"
Classified = str(OUTLocation) + str(name)+"_Classification.tif"
Confidence = str(OUTLocation) + str(name)+"_ClassConf.tif"
arcpy.gp.MLClassify_sa(List, Signature, ClassifiedSCR, "0.0", "EQUAL", "", ConfidenceSCR)
arcpy.Clip_management(ClassifiedSCR, "#" , Classified, Shp, '#', 'ClippingGeometry', 'MAINTAIN_EXTENT')
arcpy.Clip_management(ConfidenceSCR, "#" , Confidence, Shp, '#', 'ClippingGeometry', 'MAINTAIN_EXTENT')

addlayer = arcpy.mapping.Layer(Classified)
arcpy.mapping.AddLayer(df, addlayer, "TOP")
arcpy.RefreshActiveView()
arcpy.AddColormap_management(Classified, "#", clr)
arcpy.RefreshActiveView()
# Process: Create Pivot Table
outValidatedPoints = str(SCRAPLocation)+ name + "_Validation_Points.shp"
arcpy.sa.ExtractValuesToPoints(outValidationPoints,Classified ,outValidatedPoints,"NONE","VALUE_ONLY")

Freqout = str(SCRAPLocation)+ str(name) + '_Frequency.dbf'
arcpy.Frequency_analysis(outValidatedPoints, Freqout, 'Vegtype;RASTERVALU', '#')

PivotOut = str(OUTLocation)+ str(name) + '_Pivot_Table.dbf'
arcpy.PivotTable_management(Freqout, 'RASTERVALU', 'Vegtype', 'FREQUENCY', PivotOut)
addtable = arcpy.mapping.TableView(PivotOut)
arcpy.mapping.AddTableView(df, addtable)


#Process: Creates FISHNET
desc = arcpy.Describe(Shp)
FNOUT = str(name) + "Juniper_Phase_Orig.shp"
FNCLP = str(OUTLocation) + str(name) + "_Juniper_Phase.shp"
# Set the origin of the fishnet
origin_coord = str(desc.extent.XMin) + " " + str(desc.extent.YMin)
# Set the orientation
y_axis_coord = str(desc.extent.XMin) + " " + str(desc.extent.YMin + 10)
cellSize = '63.614808' 
# Number of rows and columns together with origin and opposite corner 
numRows = ""
numColumns = ""
oppositeCorner = str(desc.extent.XMax) + " " + str(desc.extent.YMax)
# Create a point label feature class 
labels = 'LABELS'
templateExtent = "#"
# Each output cell will be a polygon
geometryType = 'POLYGON'
arcpy.CreateFishnet_management(FNOUT,origin_coord, y_axis_coord, cellSize, cellSize, numRows, numColumns, oppositeCorner, labels, templateExtent, geometryType)
arcpy.Clip_analysis(FNOUT, Shp, FNCLP, '#')
FNOUT = FNCLP


# Process: Extract by Attributes
JunipEXt = str(OUTLocation) + str(name)+ "_Juniper.tif"
arcpy.gp.ExtractByAttributes_sa(ClassifiedSCR, "VALUE = 5", JunipEXt)
arcpy.AddColormap_management(JunipEXt, "#", clr)
addlayer = arcpy.mapping.Layer(JunipEXt)
arcpy.mapping.AddLayer(df, addlayer, "TOP")


# Process: Zonal Statistics as Table 
ZonalSt_Fishnet2 = str(SCRAPLocation) + str(name)+ "ZonalSt_Fishnet_2.dbf"
arcpy.gp.ZonalStatisticsAsTable_sa(FNOUT, "FID", Classified, ZonalSt_Fishnet2, "DATA", "ALL")


# Process: Zonal Statistics as Table
ZonalSt_Fishnet1 = str(SCRAPLocation) + str(name)+ "ZonalSt_Fishnet_1.dbf"
arcpy.gp.ZonalStatisticsAsTable_sa(FNOUT, "FID", JunipEXt, ZonalSt_Fishnet1, "DATA", "ALL")


# Process: Join Field
arcpy.JoinField_management(FNOUT, "FID", ZonalSt_Fishnet2, "FID_", "COUNT")
arcpy.JoinField_management(FNOUT, "FID", ZonalSt_Fishnet1, "FID_", "COUNT")


# Process: Add Percentage Field to fishnet
####AddField_management (in_table, field_name, field_type, {field_precision}, {field_scale}, {field_length}, {field_alias}, {field_is_nullable}, {field_is_required}, {field_domain})
arcpy.AddField_management (FNOUT, "Percentage", "SHORT", "#", "#", "#", "Percentage", "NON_NULLABLE", "NON_REQUIRED", "#")
arcpy.AddField_management (FNOUT, "Phase", "TEXT", "#", "#", "22", "Phase", "NULLABLE", "NON_REQUIRED", "#")
addlayer = arcpy.mapping.Layer(FNOUT)
arcpy.mapping.AddLayer(df, addlayer, "TOP")
arcpy.RefreshTOC()
arcpy.RefreshActiveView()
FNOUT1 = str(name) + "_Juniper_Phase"
arcpy.SelectLayerByAttribute_management(FNOUT1, 'NEW_SELECTION', '"COUNT" > 0')
####CalculateField_management (in_table, field, expression, {expression_type}, {code_block})
arcpy.CalculateField_management (FNOUT1, "Percentage", "([COUNT_1] /[COUNT])*100", "VB")


# Process: Add Phases to fishnet
arcpy.SelectLayerByAttribute_management(FNOUT1, 'NEW_SELECTION', '"Percentage" > 1 AND "Percentage" <= 15')          
arcpy.CalculateField_management(FNOUT1, 'Phase', '"Phase 1"', 'VB', '#')
arcpy.SelectLayerByAttribute_management(FNOUT1, 'NEW_SELECTION', '"Percentage" > 15 AND "Percentage" <= 45')
arcpy.CalculateField_management(FNOUT1, 'Phase', '"Phase 2"', 'VB', '#')
arcpy.SelectLayerByAttribute_management(FNOUT1, 'NEW_SELECTION', '"Percentage" > 45')
arcpy.CalculateField_management(FNOUT1, 'Phase', '"Phase 3"', 'VB', '#')
arcpy.SelectLayerByAttribute_management(FNOUT1, 'NEW_SELECTION', '"Percentage" = 0')
arcpy.CalculateField_management(FNOUT1, 'Phase', '"Less than 1 Percent"', 'VB', '#')
arcpy.SelectLayerByAttribute_management(FNOUT1, "CLEAR_SELECTION")

arcpy.CheckInExtension('Spatial')

arcpy.ApplySymbologyFromLayer_management (FNOUT1, clrFN)
arcpy.RefreshTOC()
arcpy.RefreshActiveView()

#Clears Memory
del mxd, addlayer, FNOUT, FNCLP, desc, ClassifiedSCR, ConfidenceSCR, Classified, Confidence, Signature, segoutname, JunipEXt
del outValidatedPoints, Freqout, PivotOut
#Cleans ScrapSpace 
if os.path.exists(SCRAP):
    shutil.rmtree(SCRAP)
else:
    x = 1
os.makedirs(SCRAP)


